"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = exports.taoNew = exports.flushChanges = exports.printGenHelp = void 0;
const tslib_1 = require("tslib");
const minimist = require("minimist");
const logger_1 = require("../shared/logger");
const params_1 = require("../shared/params");
const print_help_1 = require("../shared/print-help");
const workspace_1 = require("../shared/workspace");
const fs_1 = require("fs");
const fs_extra_1 = require("fs-extra");
const path = require("path");
const tree_1 = require("../shared/tree");
const chalk = require('chalk');
function throwInvalidInvocation() {
    throw new Error(`Specify the generator name (e.g., nx generate @nrwl/workspace:library)`);
}
function parseGenerateOpts(args, mode, defaultCollection) {
    const generatorOptions = params_1.convertToCamelCase(minimist(args, {
        boolean: ['help', 'dryRun', 'debug', 'force', 'interactive'],
        alias: {
            dryRun: 'dry-run',
            d: 'dryRun',
        },
        default: {
            debug: false,
            dryRun: false,
            interactive: true,
        },
    }));
    let collectionName = null;
    let generatorName = null;
    if (mode === 'generate') {
        if (!generatorOptions['_'] ||
            generatorOptions['_'].length === 0) {
            throwInvalidInvocation();
        }
        [collectionName, generatorName] = generatorOptions['_']
            .shift()
            .split(':');
        if (!generatorName) {
            generatorName = collectionName;
            collectionName = defaultCollection;
        }
    }
    else {
        collectionName = generatorOptions.collection;
        generatorName = '';
    }
    if (!collectionName) {
        throwInvalidInvocation();
    }
    const res = {
        collectionName,
        generatorName,
        generatorOptions,
        help: generatorOptions.help,
        debug: generatorOptions.debug,
        dryRun: generatorOptions.dryRun,
        force: generatorOptions.force,
        interactive: generatorOptions.interactive,
        defaults: generatorOptions.defaults,
    };
    delete generatorOptions.debug;
    delete generatorOptions.d;
    delete generatorOptions.dryRun;
    delete generatorOptions.force;
    delete generatorOptions.interactive;
    delete generatorOptions.defaults;
    delete generatorOptions.help;
    delete generatorOptions['--'];
    return res;
}
function printGenHelp(opts, schema, logger) {
    print_help_1.printHelp(`nx generate ${opts.collectionName}:${opts.generatorName}`, Object.assign(Object.assign({}, schema), { properties: Object.assign(Object.assign({}, schema.properties), { dryRun: {
                type: 'boolean',
                default: false,
                description: `Runs through and reports activity without writing to disk.`,
            } }) }), logger);
}
exports.printGenHelp = printGenHelp;
function readDefaultCollection(workspace) {
    return workspace.cli ? workspace.cli.defaultCollection : null;
}
function flushChanges(root, fileChanges) {
    fileChanges.forEach((f) => {
        const fpath = path.join(root, f.path);
        if (f.type === 'CREATE') {
            fs_extra_1.mkdirpSync(path.dirname(fpath));
            fs_1.writeFileSync(fpath, f.content);
        }
        else if (f.type === 'UPDATE') {
            fs_1.writeFileSync(fpath, f.content);
        }
        else if (f.type === 'DELETE') {
            try {
                const stat = fs_1.statSync(fpath);
                if (stat.isDirectory()) {
                    fs_extra_1.rmdirSync(fpath, { recursive: true });
                }
                else {
                    fs_1.unlinkSync(fpath);
                }
            }
            catch (e) { }
        }
    });
}
exports.flushChanges = flushChanges;
function printChanges(fileChanges) {
    fileChanges.forEach((f) => {
        if (f.type === 'CREATE') {
            console.log(`${chalk.green('CREATE')} ${f.path}`);
        }
        else if (f.type === 'UPDATE') {
            console.log(`${chalk.white('UPDATE')} ${f.path}`);
        }
        else if (f.type === 'DELETE') {
            console.log(`${chalk.yellow('DELETE')} ${f.path}`);
        }
    });
}
function taoNew(root, args, isVerbose = false) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const logger = logger_1.getLogger(isVerbose);
        return params_1.handleErrors(logger, isVerbose, () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const opts = parseGenerateOpts(args, 'new', null);
            return (yield Promise.resolve().then(() => require('./ngcli-adapter'))).invokeNew(logger, root, opts);
        }));
    });
}
exports.taoNew = taoNew;
function generate(root, args, isVerbose = false) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const logger = logger_1.getLogger(isVerbose);
        const ws = new workspace_1.Workspaces();
        return params_1.handleErrors(logger, isVerbose, () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const workspaceDefinition = ws.readWorkspaceConfiguration(root);
            const opts = parseGenerateOpts(args, 'generate', readDefaultCollection(workspaceDefinition));
            if (ws.isNxGenerator(opts.collectionName, opts.generatorName)) {
                const { schema, implementation } = ws.readGenerator(opts.collectionName, opts.generatorName);
                if (opts.help) {
                    printGenHelp(opts, schema, logger);
                    return 0;
                }
                const combinedOpts = yield params_1.combineOptionsForGenerator(opts.generatorOptions, opts.collectionName, opts.generatorName, workspaceDefinition, schema, opts.interactive);
                const host = new tree_1.FsTree(root, isVerbose, logger);
                const task = yield implementation(host, combinedOpts);
                const changes = host.listChanges();
                printChanges(changes);
                if (!opts.dryRun) {
                    flushChanges(root, changes);
                    if (task) {
                        yield task(host);
                    }
                }
                else {
                    logger.warn(`\nNOTE: The "dryRun" flag means no changes were made.`);
                }
            }
            else {
                return (yield Promise.resolve().then(() => require('./ngcli-adapter'))).generate(logger, root, opts);
            }
        }));
    });
}
exports.generate = generate;
//# sourceMappingURL=generate.js.map